%% include polycode.fmt

\section{Affine Functors}
\label{section:affine}

This section defines a particular, polymorphic class of endofunctors
and gives more concrete implementations for the initial algebras and
final coalgebras of special cases; cf.\ the categorical datatypes of
\citet{Hagino1987}.  By restricting the focus to instances where one
of the class parameters is a trivial type, we obtain a small system of
well-known elementary recursion schemes. It shall be demonstrated in
the next section that they have a surprisingly wide range of
applications in the domain of scientific modelling.

%if False

> {-# LANGUAGE MultiParamTypeClasses #-}
> {-# LANGUAGE FlexibleInstances #-}
> {-# LANGUAGE FlexibleContexts #-}
> {-# LANGUAGE EmptyDataDecls #-}

%endif

> module Affine  where
>
> import CoAlgebra
> import Data.List

This module uses a type |Natural| of unbounded natural numbers (the
nonnegative subset of |Integer|).
% It is defined in Sect.\,\ref{section:natural} in the appendix.

> import Natural

The class of affine functors in the category of sets is defined in
\citep{Hauhs2010a} as $\mathcal{A}^A_B = (A \times {-}) +
B$, for arbitrary sets $A, B$. We write |Go| and |Stop| for the
injections, respectively.

> data Aff a b x  =  Go !a x | Stop !b

The parameter positions |a| and |b| are made strict in the Haskell
implementation, in order to make it behave more similar to the
set-theoretic presentation.  Note that, in contrast, the functor
argument position |x| is explicitly non-strict to cater for a final
coalgebra.  The type constructor is extended to a functor in the
obvious way.

> instance Functor (Aff a b) where
>     fmap f (Go a x)  =  Go a (f x)
>     fmap f (Stop b)  =  Stop b

\subsection{Generic Initial Algebra and Final Coalgebra}

An invertible initial algebra can be given in terms of more familiar
type constructors.\footnote{To be pedantic, a list type that is strict
in its |head| argument would be required instead of the non-strict
|[]|. We silently assume that lists never contain |undefined| elements.}

> instance Initial (Aff a b) ([a], b) where
>     initial  (Go a (as, b))  =  (a : as, b)
>     initial  (Stop b)        =  ([], b)
>     laitini  (a : as, b)     =  Go a (as, b)
>     laitini  ([], b)         =  Stop b

In the category of sets and total functions, the initial algebra is a
proper substructure of the final coalgebra. In the category of Haskell
types and functions, they are the same.

> instance Final (Aff a b) ([a], b) where
>     lanif  =  initial
>     final  =  laitini

Note that the productivity requirement for |ana| is fulfilled by this
definition.

\subsection{Specific Cases}

We obtain simpler cases of special interest by substituting the unit
type |()| or an empty type (containing only |undefined|) for either
|a| or |b|.

> data Empty

\subsubsection{The case |a = ()|}

The case |a = ()| has an initial algebra with the carrier
|(Natural, b)|, by reading |[()]| as a unary number system.

> instance Initial (Aff () b) (Natural, b) where
>     initial  (Go () (n, b))  =  (n+1, b)
>     initial  (Stop b)        =  (0, b)
>     laitini  (0, b)          =  Stop b
>     laitini  (n+1, b)        =  Go () (n, b)

Algebras of this sort with carrier |c| can be represented as pairs of
|f :: c -> c| and |g :: b -> c|, playing the role of |Go| and |Stop|,
respectively.

> data Iter b c = Iter {  igo    ::  c -> c, istop  ::  b -> c }
>
> instance Algebraic (Aff () b) c (Iter b c) where
>     algebra (Iter f g) (Go () x)  =  f x
>     algebra (Iter f g) (Stop b)   =  g b

The unique homomorphism from an initial algebra into algebras in
|Iter| representation is an iteration operator.

> iter :: (Initial (Aff () b) q) => Iter b c -> q -> c
> iter = cata . algebra

Specialized to the initial algebra given above, it has the following
property:

< iter (Iter f g) (n, b) == f{-"\unskip^n"-} (g b)

Iteration can be performed analogously in the Kleisli category of a
monad.

> data IterM m b c = IterM {  igoM   ::  c -> m c, istopM  ::  b -> m c }
>
> instance (Monad m) => Algebraic (Aff () b) (m c) (IterM m b c) where
>     algebra (IterM f g) (Go () x)  =  f =<< x
>     algebra (IterM f g) (Stop b)   =  g b
>
> iterM ::  (Initial (Aff () b) q, Monad m) => IterM m b c -> q -> m c
> iterM = cata . algebra

\subsubsection{The case |b = ()|}

The case |b = ()| has an initial algebra with the carrier |[a]|,
by dropping the second component, which is redundant because of
strictness.

> instance Initial (Aff a ()) [a] where
>     initial      (Go a as)  =  a : as
>     initial      (Stop ())  =  []
>     laitini      []         =  Stop ()
>     laitini      (a : as)   =  Go a as

Algebras of this sort with carrier |c| can be represented by pairs of
|f :: a -> c -> c| and |e :: c|, alluding to the arguments of the
operation |foldr| on (finite) lists.

> data Fold a c = Fold { fgo :: a -> c -> c, fstop :: c }
>
> instance Algebraic (Aff a ()) c (Fold a c) where
>     algebra (Fold f e) (Go a x)   =  f a x
>     algebra (Fold f e) (Stop ())  =  e

The unique homomorphism from an initial algebra into algebras in
|Fold| representation is a fold operator.

> fold :: (Initial (Aff a ()) p) => (Fold a c) -> p -> c
> fold = cata . algebra

Specialized to the initial algebra given above, it is a well-known
list function.

< fold (Fold f e) == foldr f e

\subsubsection{The case |b = Empty|}

The case |b = Empty| has an empty initial algebra in the
category of sets, but a final coalgebra of streams (infinite lists).
In Haskell, the carrier of both is a type |Stream a|. Note that the
|Stop| case of |Aff| is unreachable because of strictness.

> data Stream a = (:>:) !a (Stream a)
>
> infixr 5 :>:
>
> instance Final (Aff a Empty) (Stream a) where
>     final      (a :>: as)  =  Go a as
>     lanif      (Go a as)   =  a :>: as

Coalgebras of this sort with carrier |c| can be represented by pairs
of |h :: c -> a| and |t :: c -> c|, alluding to the operations |head|
and |tail| on (infinite) lists.

> data Unfold a c = Unfold {  ugo    ::  c -> a,  ustop  ::  c -> c }
>
> instance Coalgebraic (Aff a Empty) c (Unfold a c) where
>     coalgebra (Unfold h t) x = Go (h x) (t x)

The unique homomorphism from a coalgebra in |Unfold| representation to
a final coalgebra is an unfold operator.

> unfold ::  (Final (Aff a Empty) t) => Unfold a c -> c -> t
> unfold = ana . coalgebra

It can be specified using the dual of the list function |foldr|.

< unfold (Unfold h t) == list2stream . unfoldr uncons
<     where uncons x = Just (h x, t x)

A specification using more well-known functions is as follows.

< unfold (Unfold h t) == list2stream . map h . iterate t

The conversion |list2stream| is itself an unfolding.

\begin{multicols}{2}

> list2stream :: [a] -> Stream a
> list2stream = unfold $ Unfold head tail

> stream2list :: Stream a -> [a]
> stream2list = unfoldr f
>     where f (h :>: t) = Just (h, t)

\end{multicols}

\subsubsection{The case |a = Empty|}

The case |a = Empty| is degenerate: Both |Go| and |Stop| are
strict in their first argument, hence |Aff Empty b| is trivially
isomorphic to |b|.

