\section{Continuous Distribution Functions}
\label{section:cdf}

This module defines a type of continuous probability distributions of
real numbers.

> module CDF where
>
> import CDF.UInterval

Each probability distribution is characterized by its |support|
(domain of nonzero probability, a possibly unbounded interval) and
cumulative distribution function |cdf|; a continuous monotone function
with greatest lower bound |0| and least upper bound |1|, such that
|cdf p y| gives the probability of the event |x < y| or equivalently
|x <= y| for a |p|-distributed random variable |x|.

> data CDF = CDF  {  support  ::  UInterval Double,
>                    cdf      ::  Double -> Double     }

The operation |eval| extends a cdf to the whole real number line by
setting the value outside the support to the respective limit.

> eval              ::  CDF -> Double -> Double
> eval (CDF s f) x  |   x `below` s  =  0
>                   |   x `above` s  =  1
>                   |   otherwise    =  f x

> surelyIn  ::  CDF -> UInterval Double -> Bool
> surelyIn  =   subinterval . support



\subsection{Simple Distributions}

> uniform      =  CDF  unit      (\ x -> x)
>
> cauchy       =  CDF  total     (\ x -> atan x / pi + 0.5)
>
> exponential  =  CDF  (from 0)  (\ x -> 1 - exp (-x))
>
> laplace      =  CDF total
>                 (\ x ->  if x < 0 then 0.5 * exp x
>                          else 1 - 0.5 * exp (-x))
>
> logistic s       | s > 0             =
>   CDF  total      (\ x -> 1 / (1 + exp (-x)))
>
> pareto a         | a > 0             =
>   CDF  (from 1)   (\ x -> 1 - x ** (-a))
>
> kumaraswamy a b  | a > 0  &&  b > 0  =
>   CDF  unit       (\ x -> 1 - (1 - x ** a) ** b)

\subsection{Transformations}

If |p| is the distribution of the random variable |x|, then |shift d
p| is the distribution of the random variable |x + d|, and |scale c p|
is the distribution of the random variable |x * c|.

> shift, scale :: Double -> CDF -> CDF
>
> shift d (CDF s f)           =
>   CDF  (CDF.UInterval.shift d s)  (f . (subtract d))
> scale c (CDF s f)  | c > 0  =
>   CDF  (CDF.UInterval.scale c s)  (f . (/ c))

