\subsection{Bisection}

This submodule implements the conversion of cdfs, given intensionally
by a function of type |Double -> Double|, to an extensional form of
type |[(Double, Double)]|. The conversion is performed by probing and
bisection, extended to the case of unbounded intervals.

> module  CDF.Bisection
>         (QTree(..), qtake, bisect, coords, rdata)
>     where

> import CDF
> import CDF.UInterval

The data structure of \emph{quantile trees} stores probed positions of
the cdf. Given a finite interval on the |y|-axis, the root of the tree
stores the |x|-value that corresponds to the interval midpoint,
whereas the subtrees specify the two halves of the interval
recursively.

> data QTree  =  QBranch {  left :: QTree, mid :: Double,
>                           right :: QTree                 }
>             |  QEmpty
>    deriving (Show)

The operation |qtake n| limits the depth of a quantile tree to at most
|n|.

> qtake :: (Integral t) => t -> QTree -> QTree

The operation |bisect del eps| constructs a quantile tree from a
cdf. It bisects until the distance between adjacent points is either
less than |del| in |x| direction, or less than |eps| in |y| direction

> bisect :: Double -> Double -> CDF -> QTree

The operation |coords| linearizes a quantile tree for the |y|-interval
$[0, 1]$ into a list of coordinate pairs, in ascending |x|-order.

> coords :: QTree -> [(Double, Double)]


The remainder of this module is algorithmically tedious and of too
little importance to document.

> data Bounds a b = Bounds (a -> b) Bool Bool a a b b

%if False

> instance (Show a, Show b) => Show (Bounds a b) where
>     showsPrec d (Bounds f b1 b2 x1 x2 y1 y2) =
>         showString open . shows (x1, y1) . showString ", " . shows (x2, y2)
>                  . showString close
>         where  open   =  if b1 then "[" else "..."
>                close  =  if b2 then "]" else "..."

%endif

> bounds f b1 b2 x1 x2 =
>   Bounds f b1 b2 x1 x2 (f x1) (f x2)

> function (Bounds f _ _ _ _ _ _) = f

> startbounds (CDF s f) =
>     case s of
>       (Nothing :..: Nothing)  ->
>            bounds f False  False   (-1)      1
>       (Just x1 :..: Nothing)  ->
>            bounds f True   False   x1        (x1 + 1)
>       (Nothing :..: Just x2)  ->
>            bounds f False  True    (x2 - 1)  x2
>       (Just x1 :..: Just x2)  ->
>            bounds f True   True    x1        x2


> splitbounds b = splitboundsAt (xmid b) b
> splitboundsAt xm (Bounds f b1 b2 x1 x2 y1 y2) =
>     let  ym          =  f xm
>          (x1', y1')  =  if b1 then (x1, y1)
>                         else (x1 - 2 * (x2 - x1), f x1')
>          (x2', y2')  =  if b2 then (x2, y2)
>                         else (x2 + 2 * (x2 - x1), f x2')
>     in (  Bounds f b1 True x1' xm y1' ym,
>           Bounds f True b2 xm x2' ym y2'  )

> xspan  (Bounds f b1 b2 x1 x2 y1 y2)  =  x2 - x1
> yspan  (Bounds f b1 b2 x1 x2 y1 y2)  =  y2 - y1
> xmid   (Bounds f b1 b2 x1 x2 y1 y2)  =  (x1 + x2) / 2
> ymid   (Bounds f b1 b2 x1 x2 y1 y2)  =  (y1 + y2) / 2

> bisect del eps cdf | del > 0 && eps > 0 =
>     bnode 0 1 $ startbounds cdf
>     where  bnode p1 p2 b =
>               let  p         =  (p1 + p2) / 2
>                    x         =  findMono del eps p b
>                    (lb, rb)  =  splitboundsAt x b
>                    l         =  bnode p1 p lb
>                    r         =  bnode p p2 rb
>               in   if xspan b < del || yspan b < eps
>                    then  QEmpty
>                    else  QBranch l x r

> findMono del eps y = loop
>     where  loop b =
>                let  x       =  xmid b
>                     (l, r)  =  splitbounds b
>                in  if xspan b < del || yspan b < eps then x
>                    else if y < function b x
>                         then  loop l
>                         else  loop r

> coords = ($ []) . linearize 0 1
>     where  linearize y1 y2 (QBranch l x r)  =
>               let  y  =  (y1 + y2) / 2
>               in   linearize y1 y l . ((x, y) :)
>                      . linearize y y2 r
>            linearize y1 y2 QEmpty           =  id

> qtake  0      _  =  QEmpty
> qtake  (n+1)  t  =
>     case t of
>       QEmpty           ->  QEmpty
>       (QBranch l m r)  ->  QBranch  (qtake n l) m
>                                     (qtake n r)

%if False

> rdata n = unlines . map line . coords . bisect (1 / fromInteger n) (1 / fromInteger n)
>     where line (x, y) = show x ++ "\t" ++ show y

%endif
