\subsection{Unbounded Intervals}

This submodule defines potentially unbounded intervals.  A totally
ordered datatype is conceptually extended with positive and negative
infinity. All intervals are closed on this extended range. Each
interval is represented as the pair of its bounds, with absence of
either specifying that the interval extends to infinity on the
respective end.

> module CDF.UInterval where

> import Control.Monad  (liftM)
> import Data.Maybe     (maybe, isNothing)

> data UInterval a = (:..:)  {  lowerBound :: Maybe a,
>                               upperBound :: Maybe a   }

> total  ::  (Ord a) => UInterval a
> total  =   Nothing :..: Nothing

> range               ::  (Ord a) => a -> a -> UInterval a
> range a b | a <= b  =   Just a :..: Just b

> unit  ::  (Ord a, Num a) => UInterval a
> unit  =   range 0 1

> from, upto  ::  (Ord a) => a -> UInterval a
> from a      =   Just a   :..:  Nothing
> upto b      =   Nothing  :..:  Just b

> scale, shift ::  (Ord a, Num a) =>
>                  a -> UInterval a -> UInterval a
> scale c (a :..: b) | c > 0  =
>   liftM (* c) a :..: liftM (* c) b
> shift d (a :..: b)          =
>   liftM (+ d) a :..: liftM (+ d) b

> below, above, inside      ::  (Ord a) =>
>                               a -> UInterval a -> Bool
> x  `below`   (a :..: b)   =   maybe False  (x <)   a
> x  `above`   (a :..: b)   =   maybe False  (x >)   b
> x  `inside`  (a :..: b)   =   maybe True   (x >=)  a &&
>                               maybe True   (x <=)  b

> subinterval :: (Ord a) =>
>                UInterval a -> UInterval a -> Bool
> subinterval (a :..: b) (c :..: d) = l && r
>     where  l = case a of
>                  Nothing  ->  isNothing c
>                  Just a'  ->  maybe True (a' >=) c
>            r = case b of 
>                  Nothing  ->  isNothing d
>                  Just b'  ->  maybe True (b' <=) d

> instance (Show a) => Show (UInterval a) where
>     showsPrec d (a :..: b) = l a . showString ", " . r b
>         where  l Nothing   =  showString "(-Inf"
>                l (Just a)  =  showString "[" . shows a
>                r Nothing   =  showString "+Inf)"
>                r (Just b)  =  shows b . showString "]"
