%% include polycode.fmt

\section{Universal Algebra and Coalgebra}
\label{section:coalgebra}

This section reviews the basics of universal (co)algebraic recursive
programming in Haskell. The reader familiar with the Squiggol approach
will recognize the techniques underlying the infamous banana and
lenses operators of \cite{Meijer1991}.

%if False

> {-# LANGUAGE MultiParamTypeClasses #-}
> {-# LANGUAGE FunctionalDependencies #-}
> {-# LANGUAGE FlexibleInstances #-}
> {-# LANGUAGE TypeSynonymInstances #-}

%endif

> module CoAlgebra where

\subsection{Endofunctors}

Consider a suitable category of Haskell types and pure, partial,
potentially non-strict Haskell functions, for instance
$\mathbf{Cppo}_\bot$ \citep{Hamana2007}. An endofunctor $T$ of this
category is a type constructor |t :: * -> *| such that there is an
|instance Functor t| that obeys the functor laws, namely a function
|fmap :: (a -> b) -> t a -> t b| with

< fmap  id       ==  id
< fmap  (f . g)  ==  fmap f . fmap g

\subsection{Algebras}

A $T$-algebra is a function |f :: t a -> a|, for some type |a| called its
\emph{carrier}.

> type Algebra t a  =  t a -> a

A $T$-algebra homomorphism between two $T$-algebras |f :: Algebra t a|
and |g :: Algebra t b| is a function |h :: a -> b| such that |h . f ==
g . fmap h|.  We allow for alternative representations of the type of
$T$-algebras.  The following type class abstracts from the actual
representation type |x|.

> class Algebraic t a x where algebra :: x -> Algebra t a
>
> instance Algebraic t a (Algebra t a) where algebra = id

A weakly initial $T$-algebra is an algebra |initial :: Algebra t a|
such that there is a homomorphism |cata| from it to every other
$T$-algebra.

> class Functor t => Initial t a | a -> t where
>     initial      :: Algebra t a
>     cata         :: Algebra t b -> a -> b

The homomorphism law specializes to |cata g . initial == g . fmap
(cata g)|.  \Citet{Wadler1990} has shown, by invoking
parametricity, that such a weakly initial algebra is initial proper,
in the sense that |cata g| is unique.
%%
By Lambek's lemma, the operation |initial| is an isomorphism. Its
inverse |laitini| can be implemented in terms of |initial| and |cata|
\citep{Wadler1990}, namely as |laitini == cata (fmap initial)|.
If we look at the situation the other way, and instead require an
independent implementation as method of the type class |Initial|

>     laitini  :: Coalgebra t a

that satisfies the laws |initial . laitini == id| and |laitini
. initial == id|, then we can give a generic implementation of |cata|
by applying |(. laitini)| to both sides of the initiality law:

>     cata g = h where h = g . fmap h . laitini


\subsection{Coalgebras}

The definitions of $T$-coalgebras and final $T$-coalgebras
are completely dual.

\begin{multicols}{2}

> type Coalgebra t a  = a -> t a
>
> class Coalgebraic t a x
>     where coalgebra :: x -> Coalgebra t a
>
> instance Coalgebraic t a (Coalgebra t a)
>     where coalgebra = id

> class Functor t => Final t a | a -> t where
>     final  ::  Coalgebra t a
>     lanif  ::  Algebra t a
>     ana    ::  Coalgebra t b -> b -> a
>     ana g  =   h where h = lanif . fmap h . g

\end{multicols}

The functions |final| and |lanif| are assumed to be inverses of each
other, that is |final . lanif == id| and |lanif . final == id|.
Thanks to laziness, the definition of |ana| can be productive even for
infinite data structures if the recursive invocations of |h| are
not forced by |fmap| or |lanif|, for instance by just passing them to
a non-strict data constructor.

\subsection{Generic Implementation}

The Haskell datatype mechanism allows a free generic implementation
that doubles as initial algebra and final coalgebra of certain
well-behaved functors. The exact condition is not relevant here; we
shall consider only functors that are well-behaved in the required
sense in the next section.

> newtype Fix t = In { out :: t (Fix t) }

\begin{multicols}{2}

> instance Functor t => Initial t (Fix t) where
>     initial  =  In
>     laitini  =  out

> instance Functor t => Final t (Fix t) where
>     final  =  out
>     lanif  =  In

\end{multicols}

Note that |newtype| is used instead of |data|, to the effect that both
|In| and |out| are implemented as identity. Hence the inversion and
productivity requirements are trivially satisfied.
The desired effect of separating the recursion scheme and content of
an algorithm is substantiated by the following observation:

\begin{quote}
  The definitions of |Fix|, |cata| and |ana| are the {\bfseries only} and
  {\bfseries adequate}
  instances of explicit recursion required to solve all the problems
  posed in this article!
\end{quote}
