\section{Demonstrations}

This module implements a couple of visual demonstrations on top of the
modelling framework for the logistic map. The majority of
implementation details are hidden, as GUI programming, even with the
|Chart| package, is not very illuminating.

> module Demos where

< import {-"\dots"-}

%if False

> import Natural
> import CDF
> import CDF.Bisection
> import CoAlgebra
> import Affine
> import Logistic

> import Control.Monad
> import Data.Maybe

> import Data.Colour.Names
> import Data.Colour
> import Data.Accessor
> import Graphics.Rendering.Chart
> import Graphics.Rendering.Chart.Gtk
> import Graphics.Rendering.Chart.Simple
> import Graphics.Rendering.Chart.Grid

%endif

Adjust the scale to your taste, eyesight and screen resolution.

> xscale = 300
> yscale = 250

\subsection{Demonstrations Proper}

\begin{enumerate}

\item \emph{Direct Modelling with Trajectories.} (Fig.\,\ref{fig:demoT})

> demoT :: IO ()

\begin{figure}
  \centering
  \pgfimage[width=\textwidth,interpolate]{demoT}
  \caption{Screenshot of {\em demoT}}
  \label{fig:demoT}
\end{figure}

%if False

> demoT =  demoDirect chartTrajectories
>          (replicate 5 0.35 ++ [0.7500000000001])

> demoDirect f xs = renderableToWindow (toRenderable grid) (3 * xscale) (2 * yscale)
>     where f' (x, (r_cols, title)) = f x r_cols title
>           grid = rectgrid 3 charts
>           charts = map f' $ zip xs [([(0.8, opaque red),
>                                       (1.4, opaque yellow),
>                                       (2.98, opaque turquoise)], "Stable Limit Point"),
>                                     ([(3.2, opaque red)], "2-Periodic Oscillation"),
>                                     ([(3.835, opaque red)], "3-Periodic Oscillation"),
>                                     ([(3.5, opaque red)], "4-Periodic Oscillation"),
>                                     ([(4, opaque red)], "Chaos (Dense)"),
>                                     ([(4, opaque red)], "Chaos (Divergent)")]

%endif

Trajectories of the logistic map with different initial states and
parameter values are plotted for 100 steps, showing six classes of
behaviour. From top-left to bottom-right:

\begin{enumerate}

\item Stable limit points that attract the initial state, either
one-sided ($r < 2$) or with overshooting ($r > 2$). These correspond
to the initial branch of the logistic map's bifurcation diagram.

\item Stable periodic solution of period length $2$ that attracts the
initial state; corresponds to the first bifurcation at $r = 3$.

\item Stable periodic solution of period length $2$ that attracts the
initial state; more difficult to find in the bifurcation diagram in a
small window of orderly behaviour among chaos.

\item Stable periodic solution of period length $4$ that attracts the
initial state; corresponds to the second bifurcation at $r \approx 3.45$.

\item Chaotic behaviour at $r = 4$, ranging over the whole
domain. Illustrates one of the defining properties of chaos: orbits
(points connected by a trajectory) are \emph{dense}.

\item Chaotic behaviour at $r = 4$, starting from a tiny neighbourhood
of an unstable limit point. Illustrates another defining property of
chaos: near trajectories \emph{diverge exponentially}.

\end{enumerate}

The initial state is |0.35| in all cases except the last, where it is
|0.7500000000001|.

\item \emph{Direct Modelling with Intervals.} (Fig.\,\ref{fig:demoI})

> demoI :: IO ()

\begin{figure}
  \centering
  \pgfimage[width=\textwidth,interpolate]{demoI}
  \caption{Screenshot of {\em demoI}}
  \label{fig:demoI}
\end{figure}

%if False

> demoI =  demoDirect chartIvlTrajectories
>          (replicate 5 (0.3,0.4) ++ [(0.75,0.7500000000001)])

%endif

The same six situations as above, but now with an interval of
potential initial states. Note that periodic solutions are either
self-focussing or self-dispersing, for one-sided and overshooting
convergence, respectively. Note also how quickly the state becomes
completely indeterminate when chaos emerges. The initial intervals are
$[0.3, 0.4]$ in all cases except the last, where it is $[0.75,
0.7500000000001]$.

\item \emph{Direct Modelling with Distributions.} (Fig.\,\ref{fig:demoD})

> demoD :: IO ()

\begin{figure}
  \centering
  \pgfimage[width=0.67\textwidth,interpolate]{demoD}
  \caption{Screenshot of {\em demoD}}
  \label{fig:demoD}
\end{figure}

%if False

> demoD = renderableToWindow (toRenderable grid) (2 * xscale) (2 * yscale)
>     where grid    =  rectgrid 2 charts
>           charts  =  [ chart (plotCdfs r cdf colors)
>                        ("Distribution: r =" ++ show r)
>                      | (r, cdf) <- [  (1.5, uniform), (3.2, uniform), (4, uniform),
>                                       (4, shift (1/3) (scale (1/3) uniform))]]
>           colors  =  [  opaque blue, opaque cyan, opaque green, opaque yellow,
>                         opaque orange, opaque red]

%endif

Trajectories of continuous state distribution are plotted for 5 steps
as differently coloured cdf graphs. Colours range over the rainbow, in
order of increasing wavelength.  The initial distribution is uniform
on the whole domain in all cases except the last, where it is uniform
on the middle third only.

The distributions appear to converge in all cases. For $r = 4$ (bottom
row), the limiting distribution has been studied thoroughly: it is the
Beta$(1/2, 1/2)$ distribution. Note that the initial distribution
hardly matters in this case; it converges quickly and almost surely.

\item \emph{Inverse Modelling with Observations.} (Fig.\,\ref{fig:demoR})

> demoR :: IO ()

\begin{figure}
  \centering
  \pgfimage[width=0.67\textwidth,interpolate]{demoR}
  \caption{Screenshot of {\em demoR}}
  \label{fig:demoR}
\end{figure}

%if False

> demoR = renderableToWindow (toRenderable grid) (2 * xscale) (2 * yscale)
>     where grid    =  rectgrid 2 charts
>           charts  =  [ chart [  toPlot (PlotHidden [0, toDouble $ length dat] []),
>                                 plotIvlInverse r dat (0,1) (opaque red)]
>                        ("Distribution: r =" ++ show r)
>                      | r <- [1, 3, 3.24, 4]]
>           dat     =  replicate 20 True


%endif

Successively narrowed intervals are plotted. They are obtained by
iterating the inference step for the observation ``the next state is
in the upper half'' (has colour |True|) 20 times, starting with a
completely indeterminate state. Four types of behaviour appear, from
top-left to bottom-right:

\begin{enumerate}
\item The inference fails after one step, because there is no such state.
\item The inference stagnates, because the observation is not unique.
\item The inference converges slowly.
\item The inference converges quickly.
\end{enumerate}

\item \emph{Inverse Modelling with Input.} (Fig.\,\ref{fig:demoA})

> demoA :: IO ()

\begin{figure}
  \centering
  \pgfimage[width=0.67\textwidth,interpolate]{demoA}
  \caption{Screenshot of {\em demoA}}
  \label{fig:demoA}
\end{figure}

%if False

> demoA = renderableToWindow (toRenderable grid) (2 * xscale) (2 * yscale)
>     where grid = rectgrid 2 charts
>           charts = [ chart [toPlot $ plotLine (points p) (opaque red) "r = 4"]
>                        (show p ++ "-Periodic Inverse")
>                    | p <- [1, 3, 5, 17]]
>           points = zip [0..100 :: Int] . stream2list . offline 4 0.37 . cyclo

%endif

Backwards trajectories for the logistic map with $r = 4$ are plotted
for 100 steps. In the forward mode, the map is completely chaotic for
this parameter value, i.e., periodic of all lengths exist, but are
unstable. In the backwards mode, these solutions become stable such
that most initial states are attracted to them. By the simple
expedient of driving the reverse map with a periodic stream of binary
input of the desired period length, convergence towards a solution of
that period length is effected. Running forwards from a state obtained
by this procedure, the appearance of a periodic trajectory is
produced. Of course, it will break down eventually because of chaos,
but the longer the backwards search procedure, the longer the
appearance can be maintained. Simple cases of length 1 and 3 are
shown, as well as for 5 and 17. Those are virtually impossible to
construct in the forward mode.

\end{enumerate}

%if False

\subsection{Demonstration Construction}


> rectgrid k = aboveN . map besideN . chop k . map (flip tspan (1, 1))
>     where chop k [] = []
>           chop k l = take k l : chop k (drop k l)


\subsection{Plot Layout}

> chart plots title = layout 
>     where
>     layout = layout1_title ^= title
>              $ layout1_left_axis ^: laxis_override ^= axisGridHide
>              $ layout1_right_axis ^: laxis_override ^= axisGridHide
>              $ layout1_bottom_axis ^: laxis_override ^= axisGridHide
>              $ layout1_plots ^= map Left plots
>              $ layout1_grid_last ^= False
>              $ defaultLayout1

> chartTrajectories x r_cols title = chart plots ("Trajectories: " ++ title)
>     where plots = toPlot (PlotHidden [] [0,1]) 
>                   : map (\ (r, col) -> plotTrajectory r x col) r_cols

> chartIvlTrajectories x r_cols title = chart plots ("Intervals: " ++ title)
>     where plots = toPlot (PlotHidden [] [0,1]) 
>                   : map (\ (r, col) -> plotIvlTrajectory r x col) r_cols


\subsection{Plot Construction}

> plotTrajectory r x col = toPlot $ plotLine points col ("r = " ++ show r)
>     where points = map (\ n -> (toDouble n, predict r n x)) [0 .. 100]

> plotIvlTrajectory r i col = joinPlot outer inner
>     where outer = toPlot $ plotFillBetween points col ("r = " ++ show r)
>           inner = plotTrajectory r (avg i) col
>           points = map (\ n -> (toDouble n, predictIvl r n i)) [0 .. 100]
>           avg (a, b) = (a + b) / 2

> plotIvlInverse r cs (a, b) col = joinPlot outer inner
>     where outer = toPlot $ plotFillBetween points col ("r = " ++ show r)
>           inner = toPlot $ plotLine (map avg' points) col ""
>           points = mapMaybe (\ n -> liftM (\ y -> (toDouble n, y)) (solveIvl r (take n cs) (a, b))) [0 .. length cs]
>           avg' (x, (a, b)) = (x, (a + b) / 2)

\subsection{Line Drawing}

> plotLine points color title =
>     plot_lines_style .> line_color ^= color
>     $ plot_lines_values ^= [points]
>     $ plot_lines_title ^= title
>     $ defaultPlotLines

> plotFillBetween points color title =
>     plot_fillbetween_style ^= solidFillStyle color
>     $ plot_fillbetween_values ^= points
>     $ plot_fillbetween_title ^= title
>     $ defaultPlotFillBetween

> plotCdf cdf = plotLine $ extend $ coords (bisect 0.001 0.001 cdf)
>     where extend ps = [(0, 0), (fst (head ps), 0)] ++ ps ++
>                       [(fst (last ps), 1), (1,1)]

> plotCdfs r cdf colors = map foo (zip [0..] colors)
>     where foo (n, c) = toPlot $ plotCdf (predictCdf r n cdf) c ("n = " ++ show n)

\subsection{Data Conversion}

> toDouble :: (Integral a) => a -> Double
> toDouble = fromInteger . toInteger

> cyclo n = list2stream $ cycle $ take n $ cycle [True, True, False]

%endif
