%% include polycode.fmt

\section{The Logistic Map}
\label{section:logistic}

This section is structured along the lines of
\citep[Sect.\,4]{Hauhs2010a}. Formal modelling scenarios are
mapped onto the (co)algebraic framework and exemplified with the
logistic map.

%if False

> {-# LANGUAGE FlexibleContexts #-}

%endif

> module Logistic where
>
> import Control.Monad  ((<=<))
> import CoAlgebra
> import Affine
> import Natural

This module additionally uses a type |CDF| of continuous distribution
functions.

> import CDF            (CDF(..), eval)
> import CDF.UInterval  (subinterval, unit)

The logistic map is defined as expected, but restricted to the
interval $[0,1]$ where its value is nonnegative. It is confined in
this interval and hence total for $r \leq 4$, and surjective and hence
(ambiguously) invertible for $r \geq 4$.

> type Param = Double
> type State = Double
> type Step a = Param -> a -> a
>
> logistic              ::  Step State
> logistic r x | r > 0  =   let  y = r * x * (1 - x)
>                           in   if 0 <= y && y <= 1 then y else error "out of range"

The definitions in this section use only a handful of stereotypic type
signatures, |Step| being the first example. The others shall be
introduced as we go, when their first instance arises.

As the primordial example of a map to a finite alphabet for the
purpose of symbolic dynamics, consider the binary partition that
halves the interval $[0,1]$. It is complementary to the logistic map
in the sense that each preserves exactly the information the other
destroys, and their pairing is injective and hence uniquely
invertible.

> color :: State -> Bool
> color x = x >= 0.5

The inverse of the logistic map has two symmetrical branches. We
define an auxiliary function |loginv| that computes the left branch
and another, |logbranch|, that selects a branch by colour.

> loginv      ::  Step State
> loginv r y  =   1/2 - sqrt (1/4 - y/r)
>
> type Obs a = Param -> State -> a
>
> logbranch        ::  Obs (Bool -> State)
> logbranch r y a  =   if a then 1 - loginv r y else loginv r y

The function |logbranch| is the inverse of the pairing |logistic| and
|color| in the following sense:

< logbranch r (logistic r x) (color x) == x

\subsection{Direct Functional Modelling}

Assuming that both the dynamics and the current state of the system
under study is known, the direct approach to functional modelling
addresses problems of \emph{prediction}. Information about future, yet
unobserved states is inferred from information about current or past,
observed states.  We have made the claim in \citep{Hauhs2010a}
that the paradigm of direct functional modelling is the functor |Aff
() b| for some type |b| that encodes predicates on the state space,
together with |Iter| as an encoding of algebraic queries.

\subsubsection{Perfect Information}

The basic prediction problem is solved by iteration of the step
function of the system applied to an initial state.

> type Predict a = Param -> Natural -> a -> a
>
> predict    ::  Predict State
> predict r  =   curry $ iter $ Iter (logistic r) id

Given a single initial state |x|, the expression |predict r n x|
computes the |n|-th successor state, according to the logistic map with
parameter |r|.

> type Future a = Param -> Natural -> State -> a
>
> predictTraj    ::  Future [State]
> predictTraj r  =   curry $ iter $ Iter (push $ logistic r) return
>     where push f s = f (head s) : s

Given a single initial state |x|, the expression |t = predictTraj r n
x| computes a reverse trajectory of |x|, according to the logistic map
with parameter |r|. Namely, |reverse t !! k| is the |k|-th successor
state.

More complex prediction problems can be solved in the same way, by
lifting the step function, here the logistic map, to a data type that
describes coarser information about the initial state.

\subsubsection{Imperfect Information: Nondeterminism}

The obvious mathematical data structure to describe coarse state
information is the powerset of the domain $[0, 1]$. In Haskell we will
have to make do with restricted, but computable classes of subsets.
As an example, we calculate the lifting of the logistic map to closed
intervals, represented by their end points:

> type Interval = (State, State)
>
> logimg :: Step Interval
> logimg r (x1, x2) = (z1, z2)
>     where  [y1, y2]  =  map (logistic r) [x1, x2]
>            z1        =  min y1 y2
>            z2        =  if color x1 == color x2 then max y1 y2 else r/4

(Draw the graph of the logistic map to see how this definition works.)
The lifted step function simply replaces the unlifted one in the
specification of iteration.

> predictIvl    ::  Predict Interval
> predictIvl r  =   curry $ iter $ Iter (logimg r) id

Given interval bounds |(a, b)|, the expression |predictIvl r n (a, b)|
computes the unique interval |(c, d)| such that |c <= y && y <= d| for
the |n|-th successor state |y = predict r n x|, if and only if |a <= x
&& x <= b|. It can demonstrate how, in the chaotic regime, minuscule
intervals are stretched very quickly; for instance:

< predictIvl 4 21 (0.33, 0.330001) == (0.0, 1.0)

%if False

For state descriptions with monadic structure, there is a canonical
lifting of the step function.

> predictM    ::  (Monad m) => Predict (m State)
> predictM r  =   curry $ iterM $ IterM (return . logistic r) id

This can be used, for instance, for enumerable sets encoded by the
lazy list monad.

%endif

\subsubsection{Imperfect Information: Probabilism}

As an example of a nontrivial description structure and lifting,
consider the space of continuous probability distributions on $[0,1]$.
The lifting of a distribution |p|, given by its support (a possibly
unbounded interval) and cumulative distribution function |eval p :: Double
-> Double|, can be calculated as follows; see
\citep{Hauhs2010a} for proof.

> logcdf ::  Step CDF
> logcdf r p | support p `subinterval` unit =   CDF unit h
>     where  h y  =  eval p (1/2 - q y) +  1 - eval p (1/2 + q y)
>            q y  =  sqrt (max (1/4 - y/r) 0)
>
> predictCdf    ::  Predict CDF
> predictCdf r  =   curry $ iter $ Iter (logcdf r) id

Given an initial distribution |p|, the expression |predictCdf r n p|
computes the state distribution after $n$ steps. With a little
auxiliary graphical code, the convergence towards stable
distributions, for instance $\mathrm{Beta}(1/2, 1/2)$ in the case
$r=4$, can be visualized.

\subsection{Inverse Functional Modelling}

If either the exact dynamics or the current state of the system under
study is not known, but there is data specifying the future states, an
inverse modelling problem is posed. Information about model parameters
or the current state are inferred from observations about future
states. For empirical applications, the observations are typically
imprecise and do not specify states uniquely.  We have claimed in
\citep{Hauhs2010a} that the paradigm of inverse functional
modelling is the functor |Aff a ()| for some observation type |a|,
together with |Fold| as an encoding of algebraic queries.

As an example of data specifying future states, consider the reverse
trace of colourings of a trajectory with a given length.

> logobs             ::  Future [Bool]
> logobs r 0      x  =   []
> logobs r (n+1)  x  =   map color $ predictTraj r n x

We consider only the case of inferring the current state. To this end,
we define a partial variant of the step function that checks for
consistency with some observation, in this case the binary colouring.

> type Partial a b = Param -> a -> b -> Maybe b
>
> logifcol                         ::  Partial Bool State
> logifcol r a x  |  color x == a  =   Just (logistic r x)
>                 |  otherwise     =   Nothing

The partial step function can be lifted, using the Kleisli composition
of the |Maybe| monad, to an algebra that folds colouring data.

> solve    ::  Partial [Bool] State
> solve r  =   fold $ Fold ((<=<) . logifcol r) Just

Given a finite list of colourings |w| of length |n|, the expression
|solve r w x| yields |Just (predict r n x)| if and only if |logobs r
n x == w|, and |Nothing| otherwise. Again, the class of subsets of
$[0,1]$, in this case the computable subsets, is too large to be
directly useful. Apart from heuristic probing, we can just state the
``if'' part of the above statement in a form amenable to testing, for
instance with QuickCheck \citep{Claessen2000}:

> solvePositive :: Future Bool
> solvePositive r n x = solve r w x == Just (predict r n x)
>     where w = logobs r n x

A more effective class of subsets is again the closed intervals. We
can give an inverse of |logimg| that calculates either half of the
preimage of an interval, selected by colouring.

> logpre :: Partial Bool Interval
> logpre r a (y1, y2)  |  y1 > r/4   =  Nothing
>                      |  otherwise  =
>     let  x1  =  loginv r y1
>          x2  =  loginv r (min y2 (r/4))
>     in   if a then Just (1 - x2, 1 - x1) else Just (x1, x2)

This function allows us to lift the inverse problem to intervals.

> solveIvl :: Partial [Bool] Interval
> solveIvl r = flip $ fold . Fold ((=<<) . logpre r) . Just

Given a finite list of colourings |w| of length |n| and interval
|(a, b)|, the expression |solveIvl r w (a, b)| yields the unique
interval |(c, d)| such that |c <= y && y <= d|, if and only if |y ==
predict r n x| for some |x| with |a <= x && x <= b| and |logobs r n x
== w|.

\subsection{Interactive Modelling}

So far we have only considered functional approaches to modelling
where state is studied as the cause of behaviour. Now the perspective
is turned around, thus enabling behavioural and interactive
approaches. Not surprisingly, this move coincides with the move from
algebra (|cata|) to coalgebra (|ana|). We have claimed in
\citep{Hauhs2010a} that the paradigm of behavioural modelling
is the functor |Aff a Empty| for some observation type |a|, together
with |Unfold| as an encoding of coalgebraic evaluation.

> classify    ::  Obs (Stream Bool)
> classify r  =   unfold $ Unfold color (logistic r)

For $r \leq 4$, the expression |classify r x| unfolds to an infinite
binary stream (for $r > 4$, it eventually aborts with an out-of-range
error almost surely). The resulting objects can then be studied for
similarity according to finite prefixes of those streams,
corresponding to exponentially shrinking neighbourhoods for some
suitable metric. This modelling approach, true to the paradigm of
symbolic dynamics, can be found in \citep{Rutten2000}, presented as a
coalgebraic specification in the category of complete metric
spaces. It is by generalization of that idea that our work in
\citep{Hauhs2010a} has been conceived.

By simply running the dynamics backwards, the stream of colourings is
turned from an output of an \emph{autonomous} system to the input of a
\emph{reactive} system, more adequate for living systems that are
mostly governed by the (re)actions of their inhabitants.

> offline    ::  Obs (Stream Bool -> Stream State)
> offline r  =   curry $ unfold $ Unfold fst back
>     where back (y, a :>: as) = (logbranch r y a, as)

The above model introduces an external decision-making entity, but is
not yet fully interactive, because there is no provision for feedback
of the outcome of past decisions into the stream of future decisions;
hence the name |offline|. For a fully interactive recursion scheme, a
slightly more complicated functor than those considered before is
needed. The reader may recognize it as the signature of Moore automata
with input type |u| and output type |i|. We shall use it to model a
two-player game from the perspective of the first player.

> data Game u i x = Game { imove :: i, iwait :: u -> x }
>
> instance Functor (Game u i) where
>     fmap f (Game m w) = Game m (f . w)

Truly interactive modelling scenarios, as required for \emph{adaptive}
living systems, can be expressed as \emph{hylomorphisms} (|ana|
followed by |cata|, or \emph{envelopes} in the nomenclature of
\citet{Meijer1991}) of this functor. Note that, unlike the
previous subsections, this technique requires the coincidence of
initial algebra and final coalgebra; it is sound in non-strict Haskell
but not in plain set theory.

> online    ::  Obs (Fix (Game Bool State))
> online r  =   ana iplay
>     where iplay y = Game y (logbranch r y)

The expression |online r y| denotes a game where the first player
announces the post-state |y| and moves to one of the two pre-states
according to |logbranch r y a|, depending on the input of a colour bit
|a| from the second player. The parameter |r| remains hidden.

To actually play the game, the action of the second player is wrapped
in a monadic computation. The alternating execution of half-turns can
be defined as the catamorphism of a single full-turn.

> runGame    ::  (Monad m, Initial (Game u i) g) => (i -> m u) -> g -> m ()
> runGame f  =   cata uplay
>     where uplay g = f (imove g) >>= iwait g

A simple monadic interface for the second player is console I/O.

> shell  ::  (Read u, Show i) => i -> IO u
> shell  =   (>> readLn) . print

The expression |runGame shell $ online r y| runs the game. Here is a
suggestion how to play it: choose |r = 4| for maximum freedom of
possibilities. Start with some state, say |y = 0.3|, and find a
sequence of binary choices that takes you as close to another state,
say |x = 0.4|, as possible. (Hint: You might cleverly use some of the
previous code to cheat.) For instance, the following sequence of moves
will take you within |0.001|:

< [False,False,False,True,True,False,True,False]
