% include polycode.fmt

\section{Unbounded Natural Numbers}
\label{section:natural}

This module defines a datatype of unbounded natural numbers
(non-negative integers). It is inspired by the corresponding datatype
of the Haskell-like functional-logic programming language Curry, but
reimplemented from scratch. The module interface is completely
abstract: All external access to the type is via the type classes it
implements, namely |Eq|, |Ord|, |Show|, |Read|, |Num|, |Enum|, |Real|
and |Integral|.

> module Natural (Natural) where

%if False

> import Control.Monad  (liftM)
> import Data.Maybe     (fromJust)

%endif

A natural number is either zero or a positive number.

> data Natural  =  Zero
>               |  Pos !Binary
>     deriving (Eq)

Positive numbers are encoded in binary. The number one is the base
case. The two recursive cases distinguish numbers greater than one by
their \emph{least} significant digit.

> data Binary  =  One
>              |  Zero' !Binary
>              |  One' !Binary
>     deriving (Eq)

The total order of natural numbers is defined in the obvious way by
case distinction.

> instance Ord Natural where
>     compare  Zero     Zero     =  EQ
>     compare  Zero     n        =  LT
>     compare  n        Zero     =  GT
>     compare  (Pos m)  (Pos n)  =  compare m n

For positive numbers, the most significant bit has highest impact on
their order. Hence, they need to be traversed fully, propagating a
default ordering. The actual decision is made bottom-up by an
auxiliary operation |compare'|.

> instance Ord Binary where
>     compare  One        One        =  EQ
>     compare  n          One        =  GT
>     compare  One        n          =  LT
>     compare  (Zero' m)  (Zero' n)  =  compare' EQ m n
>     compare  (Zero' m)  (One' n)   =  compare' LT m n
>     compare  (One' m)   (Zero' n)  =  compare' GT m n
>     compare  (One' m)   (One' n)   =  compare' EQ m n
>
> compare' o m n =  case compare m n of
>                     LT  ->  LT
>                     EQ  ->  o
>                     GT  ->  GT

Instances of |Show| and |Read| can be derived by specialization of the
corresponding instances for |Integer| to the non-negative subset.

> instance Show Natural where
>    showsPrec d = showsPrec d . toInteger
>
> instance Read Natural where
>    readsPrec d s =
>     do  (n, t) <- readsPrec d s
>         if n >= 0  then return (fromInteger n, t)
>                    else []

The implementation of basic arithmetic operations is rather complex,
due to the large number of cases.

> instance Num Natural where

The conversion from |Integer| works by recursive division by two.

>     fromInteger  0            =  Zero
>     fromInteger  n  | n >= 0  =  Pos $ loop n
>        where loop n =  if n == 1 then One
>                        else  let  (s, z) = n `divMod` 2
>                              in   if z == 0 then  Zero'  (loop s)
>                                   else            One'   (loop s)

The sign-oriented operations are trivial.

>     abs     n     =  n
>     signum  Zero  =  0
>     signum  n     =  1

Addition requires a distinction of several cases. It is best defined
in terms of an auxiliary operation |succb| that computes the successor
of a positive number and two operations |double| and |double'| that
compute |2 * n| and |2 * n + 1|, respectively, for input |n|.

>     Zero           +  n              =  n
>     n              +  Zero           =  n
>     Pos One        +  Pos n          =  Pos (succb n)
>     Pos n          +  Pos One        =  Pos (succb n)
>     Pos (Zero' m)  +  Pos (Zero' n)  =
>       double   (Pos m + Pos n)
>     Pos (Zero' m)  +  Pos (One' n)   =
>       double'  (Pos m + Pos n)
>     Pos (One' m)   +  Pos (Zero' n)  =
>       double'  (Pos m + Pos n)
>     Pos (One' m)   +  Pos (One' n)   =
>       double   (Pos m + Pos (succb n))

Since subtraction |(-)| is not totally defined on natural numbers, it
is best defined in terms of a partial variant |(-?)| that indicates
success or failure in the |Maybe| monad.

>     m - n     =  case m -? n of
>                    Just d   ->  d
>                    Nothing  ->  error "underflow"

Rather surprisingly, multiplication is far simpler than addition.

>     Zero           *  n     =  Zero
>     n              *  Zero  =  Zero
>     Pos One        *  n     =  n
>     Pos (Zero' m)  *  n     =  double (Pos m * n)
>     Pos (One' m)   *  n     =  double (Pos m * n) + n

Like |Show| and |Read|, instances for |Enum| and |Real| are inherited
from |Integer|.

> instance Enum Natural where
>     fromEnum  n  =  fromEnum     (toInteger n)
>     toEnum    n  =  fromInteger  (toEnum n)
>
> instance Real Natural where
>     toRational n = toRational (toInteger n)

An instance for |Integral| requires conversion back to |Integer| and
integral division.

> instance Integral Natural where

The inverse conversion is trivial.

>     toInteger Zero             =  0
>     toInteger (Pos One)        =  1
>     toInteger (Pos (Zero' n))  =  2 * toInteger (Pos n)
>     toInteger (Pos (One' n))   =  2 * toInteger (Pos n) + 1

Integral division is, of course, essentially inverse multiplication,
but requires some additional care to avoid undefined substractions
where the term |+n| is inverted. Again, the partial subtraction
operation |(-?)| comes in handy.

>     n              `quotRem`  Zero  =
>       error "division by zero"
>     Zero           `quotRem`  n     =  (0, 0)
>     Pos One        `quotRem`  n     =  (0, 1)
>     Pos (Zero' m)  `quotRem`  n     = 
>       let  (q, r) = Pos m `quotRem` n
>       in   case double r -? n of
>              Nothing  ->  (double q, double r)
>              Just r'  ->  (double' q, r')
>     Pos (One' m)   `quotRem`   n    = 
>       let  (q, r) = Pos m `quotRem` n
>       in   case double' r -? n of
>              Nothing  ->  (double q, double' r)
>              Just r'  ->  (double' q, r')

Since there is no sign, |quotRem| and |divMod| coincide.

>     divMod = quotRem

The auxiliary operation used above are defined unsurprisingly as
follows.

> succb            ::  Binary -> Binary
> succb One        =   Zero' One
> succb (Zero' n)  =   One' n
> succb (One' n)   =   Zero' (succb n)

> double, double'  ::  Natural -> Natural
>
> double Zero      =   Zero
> double (Pos n)   =   Pos (Zero' n)
>
> double' Zero     =   Pos One
> double' (Pos n)  =   Pos (One' n)

> (-?) :: Natural -> Natural -> Maybe Natural
> Zero           -?  Zero           =  Just Zero
> Zero           -?  n              =  Nothing
> Pos m          -?  Pos One        =  Just $ predb m
> Pos One        -?  Pos n          =  Nothing
> Pos (Zero' m)  -?  Pos (Zero' n)  =
>   liftM double   (Pos m -? Pos n)
> Pos (Zero' m)  -?  Pos (One' n)   =
>   liftM double'  (Pos m -? Pos (succb n))
> Pos (One' m)   -?  Pos (Zero' n)  =
>   liftM double'  (Pos m -? Pos n)
> Pos (One' m)   -?  Pos (One' n)   =
>   liftM double   (Pos m -? Pos n)

The operation |predb| required by |(-?)| is conceptually the inverse
of |succb|, but returns a |Natural| instead of a |Binary|.

> predb            ::  Binary -> Natural
> predb One        =   Zero
> predb (Zero' n)  =   double' (predb n)
> predb (One' n)   =   Pos (Zero' n)
